using System;
using System.Configuration.Install;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.IO;
using System.Xml;

namespace BE.pinvoke
{
	[RunInstaller(true)]
	[ComVisible(false)]
    public class VS2005AddinInstaller : Installer
	{
		public override void Install(System.Collections.IDictionary
			stateSaver)
		{
			base.Install(stateSaver);

            try
            {
                bool allUsers = (this.Context.Parameters["ALLUSERS"] == "1");

                string targetPath; string targetLocation;
                if (allUsers)
                {
                    targetPath = CommonAddInPath;
                    targetLocation = CommonAddInLocation;
                }
                else
                {
                    targetPath = UsersCommonAddInPath;
                    targetLocation = UsersCommonAddInLocation;
                }

                if (!Directory.Exists(targetPath))
                    Directory.CreateDirectory(targetPath);

                XmlDocument doc = new XmlDocument();
                doc.Load(AddInDefaultLocation);
                XmlNodeList nl = doc.GetElementsByTagName("Assembly");
                nl[0].InnerText = AssemblyLocation;
                doc.Save(targetLocation);

                File.Delete(AddInDefaultLocation);
            }
            catch (Exception ex)
            {
                throw new InstallException("Failed to register AddIn", ex);
            }
		}

        public override void Rollback(System.Collections.IDictionary savedState)
        {
            base.Rollback(savedState);

            CleanUp();
        }

		public override void Uninstall(System.Collections.IDictionary
			savedState)
		{
			base.Uninstall(savedState);

            CleanUp();
		}

        void CleanUp()
        {
            try
            {
                if (File.Exists(CommonAddInLocation))
                    File.Delete(CommonAddInLocation);
                if (File.Exists(UsersCommonAddInLocation))
                    File.Delete(UsersCommonAddInLocation);
            }
            catch (Exception ex)
            {
                throw new InstallException("Operation failed!", ex);
            }
        }

        public string AddInDefaultLocation
        {
            get
            {
                string path = Path.GetDirectoryName(AssemblyLocation);
                string af = Path.Combine(path, AddInFilename );
                return af;
            }
        }

        public string AssemblyLocation
        {
            get
            {
                string f = this.GetType().Assembly.Location;
                return f;
            }
        }

        public string AddInFilename
        {
            get
            {
                return "BE.pinvoke.AddIn";
            }
        }

        public string CommonAddInPath
        {
            get
            {
                // Problem on German Windows XP - Thanxs to AlexanderGross
                //string commonApplicationData = Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData);
                string commonApplicationData = Path.Combine(
                    System.Environment.GetEnvironmentVariable("ALLUSERSPROFILE")
                    , "Application Data");
                string addinPath = Path.Combine(commonApplicationData, @"Microsoft\MSEnvShared\Addins");
                return addinPath;
            }
        }

        public string CommonAddInLocation
        {
            get
            {
                return Path.Combine(CommonAddInPath, AddInFilename);
            }
        }

        public string UsersCommonAddInPath
        {
            get
            {
                string usersCommonApplicationData = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                string addinPath = Path.Combine(usersCommonApplicationData, @"Microsoft\MSEnvShared\Addins");
                return addinPath;
            }
        }

        public string UsersCommonAddInLocation
        {
            get
            {
                return Path.Combine(UsersCommonAddInPath, AddInFilename);
            }
        }
	}
}
